#!/usr/bin/python3

# Obtained from https://gitlab.kitware.com/cmake/cmake/-/issues/21347

import sys
import networkx as nx
import re


def unionizeCycle(cycle, unions):
    me = set(cycle)
    others = list()
    for union in unions:
        if me.intersection(union):
            me.update(union)
        else:
            others.append(set(union))
    others.append(me)
    return others


def unionizeCycles(cycles):
    unionized = set()
    for cycle in cycles:
        unionized = unionizeCycle(cycle, unionized)
    return unionized


def updateLine(line, cycleForNode):
    PATT = re.compile(r'"(node\d+)" -> "(node\d+)"')
    m = PATT.search(line)
    if not m:
        return line
    a = m.group(1)
    b = m.group(2)
    clusterA = None
    clusterB = None
    if a in cycleForNode:
        clusterA = cycleForNode[a]
    if b in cycleForNode:
        clusterB = cycleForNode[b]
    if clusterA == clusterB:
        return line

    if clusterA is not None:
        line = re.sub(a, 'Cycle ' + str(clusterA), line)
    if clusterB is not None:
        line = re.sub(b, 'Cycle ' + str(clusterB), line)

    return line


def main():
    if len(sys.argv) > 1:
        path_in = sys.argv[1]
        path_out = sys.argv[2]
    else:
        sys.exit(1)

    # read in the specified file, create a networkx DiGraph
    G = nx.DiGraph(nx.nx_pydot.read_dot(path_in))

    C = nx.simple_cycles(G)
    # C = nx.cycle_basis(G)
    allCycles = list(C)
    allCycles = list(unionizeCycles(allCycles))
    allCycles = sorted(allCycles, key=len, reverse=True)

    cycleForNode = dict()

    with open(path_in, 'r') as filIn, open(path_out, 'w') as filOut:

        # 'strict digraph' deduplicates edges, but keeps only the first occurence.
        # So if module A depends privately on B and publicly on C,
        # B and C are in cycle,
        # it witll be shown as if A depends on cycle only publicly
        filOut.write("""strict digraph "GG" {
node [
  fontsize = "12"
];
compound=true
""")
        for i, cycle in enumerate(allCycles):

            filOut.write('subgraph cluster_%d {' % i)
            filOut.write('label="Cycle %d (%d members)";' % (i, len(cycle)))
            filOut.write('node [style=filled];')
            filOut.write('color=blue')
            for node in cycle:
                filOut.write('"%s"' % node)
                cycleForNode[node] = i
            filOut.write('}')
        # skip 4 line header
        next(filIn)
        next(filIn)
        next(filIn)
        next(filIn)
        for line in filIn:
            filOut.write(updateLine(line, cycleForNode))


if __name__ == "__main__":
    main()
